package cn.skcks.study.springcloud.project01;

import cn.skcks.study.springcloud.project01.config.SeaweedFsConfig;
import io.minio.BucketExistsArgs;
import io.minio.MakeBucketArgs;
import io.minio.MinioClient;
import io.minio.PutObjectArgs;
import io.minio.messages.Bucket;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.NONE)
@ExtendWith(SpringExtension.class)
@Slf4j
public class NacosConsumer01Test {
    public static void main(String[] args) {
        SpringApplication.run(NacosConsumer01Test.class,args);
    }
    private SeaweedFsConfig seaweedFsConfig;
    private MinioClient mc;
    private final static String bucket = "simple-test";

    @Autowired
    public void setSeaweedFsConfig(SeaweedFsConfig seaweedfsConfig) {
        this.seaweedFsConfig = seaweedfsConfig;
        log.info("{}",seaweedfsConfig);
        this.mc = MinioClient.builder()
                .endpoint(seaweedfsConfig.getUrl())
                .credentials(seaweedfsConfig.getAccessKey(),seaweedfsConfig.getSecretKey())
                .build();
    }

    @Test
    void checkBucket() throws Exception {
        if(!mc.bucketExists(BucketExistsArgs.builder().bucket(bucket).build())){
            mc.makeBucket(MakeBucketArgs.builder().bucket(bucket).build());
        }

        for(Bucket buckets:mc.listBuckets()){
            log.info("{}",buckets.name());
        }
    }

    @Test
    void upload() throws Exception{
        checkBucket();
        String f = "D:\\Temp\\temp.zip";
        Path path = Paths.get(f);
        try(InputStream is = Files.newInputStream(path)){
            mc.putObject(PutObjectArgs.builder()
                    .bucket(bucket)
                    .object(path.getFileName().toString())
                    .stream(is,-1,1024 * 1024 * 1024)
                    .build());
        }

    }

    @Test
    void uploadZipFile() throws Exception {
        checkBucket();
        String path = "D:\\Temp\\temp.zip";
        try(ZipFile zipFile = new ZipFile(path)){
            ZipInputStream zipInputStream = new ZipInputStream(new BufferedInputStream(Files.newInputStream(Paths.get(path))));
            ZipEntry zipEntry;
            while ((zipEntry = zipInputStream.getNextEntry()) != null) {
                String fileName = zipEntry.getName();
                long size = zipEntry.getSize();
                log.info("{} {}",fileName,size);
                mc.putObject(PutObjectArgs.builder()
                        .bucket(bucket)
                        .object(fileName)
                        .stream(zipFile.getInputStream(zipEntry),size,-1)
                        .build());
            }
        }
    }
}
